@@ -28,6 +28,9 @@
 /* Undefine if <netdb.h> contains this, otherwise define to 1 */
 #undef NI_MAXHOST
 
+/* Undefine if <netdb.h> contains this, otherwise define to 32 */
+#undef NI_MAXSERV
+
 /* Take care of getaddrinfo */
 #undef HAVE_STRUCT_ADDRINFO
 #ifndef AI_PASSIVE
