@@ -41,6 +41,9 @@
 #include "config.h"
 #include "scanssh.h"
 
+extern int ssh_sendident;
+extern char *ssh_ipalias;
+extern int port;
 int alarmed;
 
 void
@@ -52,60 +55,10 @@ sigalrm_handler(int sig)
 }
 
 int
-scanhost(struct argument *arg, char *buf, size_t size)
+scan_ssh(int sock, char *buf, size_t size)
 {
-	int res, sock, j;
-	struct addrinfo hints, *ai;
+	int j;
 	char firstline[255];
-	char ntop[NI_MAXHOST];
-	
-	switch (arg->a_type) {
-	case AF_INET:
-		ipv4toa(ntop, sizeof(ntop), &arg->a_ipv4);
-		break;
-	default:
-		strlcpy(buf, "<unsuppfamily>", size);
-		return (-1);
-	}
-
-	sock = socket(arg->a_type, SOCK_STREAM, 0);
-	if (sock == -1) {
-		strlcpy(buf, "<socketcreate>", size);
-		return (-1);
-	}
-
-	memset(&hints, 0, sizeof(hints));
-        hints.ai_family = AF_UNSPEC;
-        hints.ai_socktype = SOCK_STREAM;
-        if (getaddrinfo(ntop, "22", &hints, &ai) != 0) {
-		strlcpy(buf, "<getaddrinfo>", size);
-		return (-1);
-	}
-
-	alarm(CONNECTWAIT);
-	res = connect(sock, ai->ai_addr, ai->ai_addrlen);
-	alarm(0);
-	if (res == -1) {
-		close(sock);
-		switch (errno) {
-		case ETIMEDOUT:
-		case EINTR:
-#ifdef __linux__
-		case EHOSTUNREACH:
-#endif
-			strlcpy(buf, "<timeout>", size);
-			return (-1);
-		case ECONNREFUSED:
-			strlcpy(buf, "<refused>", size);
-			return (-1);
-		case ENETUNREACH:
-			strlcpy(buf, "<unreachable>", size);
-			return (-1);
-		default:
-			snprintf(buf, size, "<%s>", strerror(errno));
-			return (-1);
-		}
-	}
 	
 	/* Read other side\'s version identification. */
         for (j = 0; j < MAXITER; j++) {
@@ -122,11 +75,9 @@ scanhost(struct argument *arg, char *buf, size_t size)
 			}
 			alarm(0);
                         if (len < 0 && !j) {
-				close(sock);
 				strlcpy(buf, "<timeout>", size);
 				return (-1);
 			} else if (len != 1 && !j) {
-				close(sock);
                                 strlcpy(buf, "<closed>", size);
 				return (-1);
 			} else if (len != 1) {
@@ -150,9 +101,153 @@ scanhost(struct argument *arg, char *buf, size_t size)
 			strlcpy(firstline, buf, sizeof(firstline));
         }
 	if (j >= MAXITER)
-		strlcpy(buf, firstline, sizeof(firstline));
-	else
+		strlcpy(buf, firstline, size);
+	else if (ssh_sendident)
 		atomicio(write, sock, SSHMAPVERSION, sizeof(SSHMAPVERSION));
+
+	return (0);
+}
+
+#define HTTP_SCAN	"HEAD /index.html HTTP/1.0\n\n"
+
+int
+scan_http(int sock, char *buf, size_t size)
+{
+	int j;
+	char firstline[255];
+
+	atomicio(write, sock, HTTP_SCAN, strlen(HTTP_SCAN));
+	
+	/* Read other side\'s version identification. */
+        for (j = 0; j < MAXITER; j++) {
+		int i;
+                for (i = 0; i < size - 1; i++) {
+                        int len = -1;
+			if (!j && !i)
+				alarm (LONGWAIT);
+			else
+				alarm (SHORTWAIT);
+			while ((len = read(sock, &buf[i], 1)) == -1) {
+				if (len == -1 && errno == EINTR)
+					break;
+			}
+			alarm(0);
+                        if (len < 0 && !j) {
+				strlcpy(buf, "<timeout>", size);
+				return (-1);
+			} else if (len != 1 && !j) {
+                                strlcpy(buf, "<closed>", size);
+				return (-1);
+			} else if (len != 1) {
+				j = MAXITER;
+				break;
+			}
+			
+                        if (buf[i] == '\r') {
+                                buf[i] = 0;
+                                continue;
+                        }
+                        if (buf[i] == '\n') {
+                                buf[i] = 0;
+                                break;
+                        }
+                }
+                buf[size - 1] = 0;
+                if (strncmp(buf, "Server:", 7) == 0)
+                        break;
+		if (j == 0)
+			strlcpy(firstline, buf, sizeof(firstline));
+        }
+
+	if (j >= MAXITER)
+		strlcpy(buf, firstline, size);
+}
+
+int
+scanhost(struct argument *arg, char *buf, size_t size)
+{
+	int res, sock;
+	struct addrinfo hints, *ai;
+	char ntop[NI_MAXHOST], sport[NI_MAXSERV];
+	
+	switch (arg->a_type) {
+	case AF_INET:
+		ipv4toa(ntop, sizeof(ntop), &arg->a_ipv4);
+		break;
+	default:
+		strlcpy(buf, "<unsuppfamily>", size);
+		return (-1);
+	}
+
+	sock = socket(arg->a_type, SOCK_STREAM, 0);
+	if (sock == -1) {
+		strlcpy(buf, "<socketcreate>", size);
+		return (-1);
+	}
+	if (ssh_ipalias != NULL) {
+	    memset(&hints, 0, sizeof(hints));
+	    hints.ai_family = PF_UNSPEC;
+	    hints.ai_socktype = SOCK_STREAM;
+	    hints.ai_flags = AI_PASSIVE;
+	    res = getaddrinfo(ssh_ipalias, "0", &hints, &ai);
+	    if (res) {
+		warn("%s: getaddrinfo() failed: %d", ssh_ipalias, res);
+		close(sock);
+		return (-1);
+	    }
+	    if (bind(sock, ai->ai_addr, ai->ai_addrlen) == -1) {
+		perror(ssh_ipalias);
+		close(sock);
+		freeaddrinfo(ai);
+		return (-1);
+            }
+	    freeaddrinfo(ai);
+	}
+
+	memset(&hints, 0, sizeof(hints));
+        hints.ai_family = AF_UNSPEC;
+        hints.ai_socktype = SOCK_STREAM;
+	snprintf(sport, sizeof(sport), "%d", port);
+        if (getaddrinfo(ntop, sport, &hints, &ai) != 0) {
+		strlcpy(buf, "<getaddrinfo>", size);
+		return (-1);
+	}
+
+	alarm(CONNECTWAIT);
+	res = connect(sock, ai->ai_addr, ai->ai_addrlen);
+	alarm(0);
+	freeaddrinfo(ai);
+	if (res == -1) {
+		close(sock);
+		switch (errno) {
+		case ETIMEDOUT:
+		case EINTR:
+#ifdef __linux__
+		case EHOSTUNREACH:
+#endif
+			strlcpy(buf, "<timeout>", size);
+			return (-1);
+		case ECONNREFUSED:
+			strlcpy(buf, "<refused>", size);
+			return (-1);
+		case ENETUNREACH:
+			strlcpy(buf, "<unreachable>", size);
+			return (-1);
+		default:
+			snprintf(buf, size, "<%s>", strerror(errno));
+			return (-1);
+		}
+	}
+
+	switch(port) {
+	case 22:
+		scan_ssh(sock, buf, size);
+		break;
+	case 80:
+		scan_http(sock, buf, size);
+		break;
+	}
+
 	close(sock);
 	return (0);
 }
